<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <title>Login to RealNet</title>
    {{ bootstrap.load_css() }}
</head>
<body>
    <div class="container h-100">
        <div class="d-flex justify-content-md-center align-items-center vh-100">
            <div class="row">
                <div class="col-md-12">
                    <h3>Login to RealNet</h3>
                </div>
                <div class="col-md-12" style="margin-bottom:30px;">
                    <form action="" method="post">
                        <label for="username" class="form-label">username:</label><br>
                        <input type="text" id="username" name="username" class="form-control"><br>
                        <label for="password" class="form-label">password:</label><br>
                        <input type="password" id="password" name="password" class="form-control"><br>
                        <button type="submit" class="btn btn-primary btn-lg">Login</button>
                    </form>
                </div>
                <div class="col-md-12">

                        {% for auth in authenticators %}
                            <a href="{{auth.url}}" class="btn btn-secondary btn-sm">{{auth.name}}</a>
                        {% endfor %}
                </div>
                {% if error %}
                <div class="col-md-12">
                    <div class="alert alert-danger" role="alert">
                        {{msg}}
                    </div>
                </div>
                {% endif %}
            </div>
        </div>
    </div>
    {{ bootstrap.load_js() }}
</body>
</html>